package com.github.polpetta.mezzotre;

import com.github.polpetta.mezzotre.orm.OrmDI;
import com.github.polpetta.mezzotre.route.RouteDI;
import com.github.polpetta.mezzotre.route.Telegram;
import com.github.polpetta.mezzotre.telegram.callbackquery.CallbackQueryDI;
import com.github.polpetta.mezzotre.telegram.command.CommandDI;
import com.github.polpetta.mezzotre.util.ServiceModule;
import com.github.polpetta.mezzotre.util.UtilDI;
import com.google.inject.*;
import com.google.inject.Module;
import com.google.inject.name.Names;
import io.jooby.*;
import io.jooby.banner.BannerModule;
import io.jooby.di.GuiceModule;
import io.jooby.di.JoobyModule;
import io.jooby.ebean.TransactionalRequest;
import io.jooby.json.GsonModule;
import java.util.*;
import java.util.function.Function;

public class App extends Jooby {

  public static final Function<Jooby, Collection<Module>> DEFAULT_DI_MODULES =
      (jooby) -> {
        final HashSet<Module> modules = new HashSet<>();
        modules.add(new OrmDI());
        modules.add(new UtilDI());
        modules.add(new RouteDI());
        modules.add(new CommandDI());
        modules.add(new CallbackQueryDI());
        return modules;
      };

  public App() {
    this(Stage.PRODUCTION, null);
  }

  public App(Stage runningEnv, Collection<Module> modules) {

    Collection<Module> toInject =
        new ArrayList<>(Optional.ofNullable(modules).orElse(Collections.emptySet()));
    if (modules == null || modules.size() == 0) {
      toInject = DEFAULT_DI_MODULES.apply(this);
    }
    toInject.add(new AppDI(this));
    toInject.add(new JoobyModule(this));

    final Injector injector = Guice.createInjector(runningEnv, toInject);
    install(new GuiceModule(injector)); // We need to do it here

    install(new BannerModule("Mezzotre"));
    install(new OpenAPIModule("/docs"));
    install(new GsonModule());
    install(injector.getInstance(Key.get(Extension.class, Names.named("hikariPoolExtension"))));
    install(
        injector.getInstance(Key.get(Extension.class, Names.named("flyWayMigrationExtension"))));
    install(injector.getInstance(Key.get(Extension.class, Names.named("ebeanExtension"))));
    install(injector.getInstance(Key.get(ServiceModule.class, Names.named("serviceModule"))));

    decorator(new AccessLogHandler());
    decorator(new TransactionalRequest());

    // We need to put this there otherwise MockRouter for testing purposes won't work
    get("/", ctx -> "Hello World!");
    mvc(Telegram.class);
  }

  public static void main(final String[] args) {
    runApp(args, ExecutionMode.EVENT_LOOP, App::new);
  }
}
