package com.github.polpetta.mezzotre;

import com.github.polpetta.mezzotre.orm.BatchBeanCleanerService;
import com.google.common.util.concurrent.Service;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import io.jooby.Jooby;
import java.util.List;
import java.util.concurrent.TimeUnit;
import javax.inject.Named;
import javax.inject.Singleton;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;

public class AppDI extends AbstractModule {

  // In the future we can get this name from mvn, by now it is good as it is
  public static final String APPLICATION_NAME = "Mezzotre";
  private final Jooby jooby;

  public AppDI(Jooby jooby) {
    this.jooby = jooby;
  }

  @Provides
  public Logger getLogger() {
    return jooby.getLog();
  }

  @Named("applicationName")
  @Singleton
  @Provides
  public String getAppName() {
    return APPLICATION_NAME;
  }

  @Provides
  @Singleton
  @Named("services")
  public List<Service> getApplicationServices(
      Logger logger,
      @Named("serviceRunningCheckTime") Pair<Integer, TimeUnit> serviceRunningCheckTime) {
    return List.of(new BatchBeanCleanerService(logger, serviceRunningCheckTime));
  }
}
