package com.github.polpetta.mezzotre.route;

import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.pengrad.telegrambot.TelegramBot;
import io.jooby.Jooby;
import java.util.Optional;
import javax.inject.Singleton;

public class RouteDI extends AbstractModule {
  @Provides
  @Singleton
  public TelegramBot getTelegramBot(Jooby jooby) {
    final String telegramKey =
        Optional.ofNullable(jooby.getEnvironment().getConfig().getString("telegram.key"))
            .or(
                () ->
                    Optional.ofNullable(
                        jooby.getEnvironment().getConfig().getString("TELEGRAM_KEY")))
            .filter(s -> !s.isBlank())
            .orElseThrow(
                () ->
                    new IllegalStateException(
                        "Telegram token is required to make the application work. Please set"
                            + " 'telegram.key = \"value\"' in your application.conf file or as the"
                            + " application argument. Alternatively, you can set 'TELEGRAM_KEY' as"
                            + " an environment variable."));
    return new TelegramBot(telegramKey);
  }
}
