package com.github.polpetta.mezzotre.telegram.callbackquery;

import com.github.polpetta.mezzotre.orm.model.CallbackQueryContext;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;

/**
 * A processor is an object which is able to process an incoming Telegram {@link Update} containing
 * a {@link com.pengrad.telegrambot.model.CallbackQuery}. Any event of this type is related to a
 * previous message.
 *
 * @author Davide Polonio
 * @since 1.0
 */
public interface Processor {

  /**
   * The event name this processor is able to process
   *
   * @return a {@link String} containing the name of the event supported
   */
  String getEventName();

  // FIXME javadoc
  default boolean canBeDirectlyInvokedByTheUser() {
    return false;
  }

  default Optional<String> getPrettyPrintLocaleKeyName() {
    return Optional.empty();
  }

  /**
   * Process the current event
   *
   * @param callbackQueryContext the corresponding even {@link CallbackQueryContext} previously
   *     stored in the database
   * @param update the Telegram {@link Update} containing a {@link
   *     com.pengrad.telegrambot.model.CallbackQuery} incoming Telegram
   * @return a {@link CompletableFuture} containing an {@link Optional} that may or maybe not have a
   *     {@link BaseRequest}, that is the response that will be sent back to Telegram
   */
  CompletableFuture<Optional<BaseRequest<?, ?>>> process(
      CallbackQueryContext callbackQueryContext, Update update);
}
