package com.github.polpetta.mezzotre.telegram.command;

import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.assistedinject.FactoryModuleBuilder;
import java.util.HashMap;
import java.util.Map;
import javax.inject.Named;
import javax.inject.Singleton;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.runtime.RuntimeConstants;
import org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader;

public class CommandDI extends AbstractModule {

  @Override
  protected void configure() {
    super.configure();

    install(
        new FactoryModuleBuilder()
            .implement(Processor.class, NotFound.class)
            .build(NotFoundFactory.class));
  }

  private static Map<String, Processor> mapForProcessor(Processor processor) {
    final HashMap<String, Processor> commandMap = new HashMap<>();
    processor
        .getTriggerKeywords()
        .forEach(
            keyword -> {
              commandMap.put(keyword, processor);
            });
    return commandMap;
  }

  @Provides
  @Singleton
  @Named("commandProcessor")
  public Map<String, Processor> getCommandProcessor(Start start, Help help) {
    final HashMap<String, Processor> commandMap = new HashMap<>();
    commandMap.putAll(mapForProcessor(start));
    commandMap.putAll(mapForProcessor(help));

    return commandMap;
  }

  @Provides
  public VelocityEngine getVelocityEngine() {
    final VelocityEngine velocityEngine = new VelocityEngine();
    velocityEngine.setProperty(RuntimeConstants.RESOURCE_LOADER, "classpath");
    velocityEngine.setProperty(
        "classpath.resource.loader.class", ClasspathResourceLoader.class.getName());
    velocityEngine.init();
    return velocityEngine;
  }
}
