package com.github.polpetta.mezzotre.telegram.command;

import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.google.inject.assistedinject.Assisted;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import com.pengrad.telegrambot.request.SendMessage;
import java.util.Collections;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import javax.inject.Inject;

/**
 * Generates a "Command not found" message
 *
 * @author Davide Polonio
 * @since 1.0
 * @see com.github.polpetta.mezzotre.telegram.callbackquery.NotFound for the event-based version
 */
class NotFound implements Processor {

  private final String commandName;

  @Inject
  public NotFound(@Assisted String commandName) {
    this.commandName = commandName;
  }

  @Override
  public Set<String> getTriggerKeywords() {
    return Collections.emptySet();
  }

  @Override
  public CompletableFuture<Optional<BaseRequest<?, ?>>> process(TgChat chat, Update update) {
    // FIXME complete it with: localization, callbackQuery to show help message
    return CompletableFuture.completedFuture(update)
        .thenApply(
            ignored ->
                Optional.of(
                    new SendMessage(chat.getId(), "Command " + commandName + " is not valid")));
  }
}
