package com.github.polpetta.mezzotre.telegram.command;

import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

/**
 * This interfaces provides a way to digest incoming updates that are commands, e.g. {@code
 * /this_command arg1}
 *
 * @author Davide Polonio
 * @since 1.0
 */
public interface Processor {

  /**
   * Provides the keywords to trigger this executor. Note that it must start with "/" at the
   * beginning, e.g. {@code /start}.
   *
   * @return a {@link Set} providing the keywords to trigger the current {@link Processor}
   */
  Set<String> getTriggerKeywords();

  /**
   * Process the current update
   *
   * @param chat the chat the {@link Processor} is currently replying to
   * @param update the update to process
   * @return a {@link CompletableFuture} with the result of the computation
   */
  CompletableFuture<Optional<BaseRequest<?, ?>>> process(TgChat chat, Update update);

  /**
   * Provide the key to retrieve the current processor descriptor. This is useful for help messages
   * or to provide a user with a description of what this command does. Whilst a default
   * implementation is provided, we suggest to rename it accordingly since the name generation is
   * based on reflection and can be fragile and subject to code refactor changes.
   *
   * @return a {@link String} with the name of the localization key containing the command
   *     description
   */
  default String getLocaleDescriptionKeyword() {
    return this.getClass().getSimpleName().toLowerCase() + ".cmdDescription";
  }
}
