package com.github.polpetta.mezzotre.util;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.TimeZone;

public class Clock {

  /**
   * @return the current UNIX timestamp in milliseconds (so since 1970/01/01)
   */
  public long now() {
    return System.currentTimeMillis();
  }

  public Timestamp nowInTimestamp() {
    return new Timestamp(now());
  }

  public Date nowInDate() {
    return new Date();
  }

  public String getNowDateISO8601() {
    final TimeZone tz = TimeZone.getTimeZone("UTC");
    final DateFormat df = new SimpleDateFormat("yyyy-MM-dd'Z'");
    df.setTimeZone(tz);
    return df.format(new Date());
  }

  public String getNowDateTimeISO8601() {
    final DateTimeFormatter df = DateTimeFormatter.ISO_DATE_TIME.withZone(ZoneId.of("UTC"));
    return df.format(Instant.now());
  }

  public static String getDateISO86001(long unixTimestamp) {
    final DateTimeFormatter df = DateTimeFormatter.ISO_DATE.withZone(ZoneId.of("UTC"));
    return df.format(Instant.ofEpochMilli(unixTimestamp));
  }

  public static String getDateTimeISO8601(long unixTimestamp) {
    final DateTimeFormatter df = DateTimeFormatter.ISO_DATE_TIME.withZone(ZoneId.of("UTC"));
    return df.format(Instant.ofEpochMilli(unixTimestamp));
  }

  public static Timestamp timestampFromString(String entry, String pattern) {
    final DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(pattern);
    final LocalDate parse = LocalDate.parse(entry, dateTimeFormatter);
    return new Timestamp(parse.toEpochSecond(LocalTime.MIDNIGHT, ZoneOffset.of("Z")) * 1000L);
  }

  public static Timestamp getTimestampFromUnixEpoch(long unixTimestamp) {
    return new Timestamp(unixTimestamp);
  }
}
