package com.github.polpetta.mezzotre.util;

import com.google.common.util.concurrent.MoreExecutors;
import com.google.common.util.concurrent.Service;
import com.google.common.util.concurrent.ServiceManager;
import io.jooby.Extension;
import io.jooby.Jooby;
import java.time.Duration;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeoutException;
import javax.inject.Inject;
import javax.inject.Named;
import org.jetbrains.annotations.NotNull;

// FIXME doc, tests(?)
public class ServiceModule implements Extension {

  private final ServiceManager serviceManager;

  @Inject
  public ServiceModule(@Named("services") List<Service> services) {
    serviceManager = new ServiceManager(services);
  }

  @Override
  public boolean lateinit() {
    return true;
  }

  @Override
  public void install(@NotNull Jooby application) throws Exception {
    final CompletableFuture<Void> initialization = new CompletableFuture<>();
    serviceManager.addListener(
        new ServiceManager.Listener() {
          @Override
          public void healthy() {
            super.healthy();
            application.getLog().info("All internal application services are up and running");
            initialization.complete(null);
          }

          @Override
          public void failure(@NotNull Service service) {
            super.failure(service);
            initialization.completeExceptionally(service.failureCause());
          }
        },
        MoreExecutors.directExecutor());

    Runtime.getRuntime()
        .addShutdownHook(
            new Thread(
                () -> {
                  try {
                    serviceManager.stopAsync().awaitStopped(Duration.ofSeconds(15));
                  } catch (TimeoutException ex) {
                    application
                        .getLog()
                        .warn(
                            "Unable to correctly stop all the services, got the following error"
                                + " while stopping: "
                                + ex.getMessage());
                    throw new RuntimeException(ex);
                  }
                }));

    // Blocking call to wait for all the services to be up and running
    serviceManager.startAsync();
    initialization.get();
  }
}
