package com.github.polpetta.mezzotre;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.mock;

import com.github.polpetta.mezzotre.util.ServiceModule;
import com.google.inject.AbstractModule;
import com.google.inject.Module;
import com.google.inject.Provides;
import com.google.inject.Stage;
import io.jooby.*;
import io.jooby.ebean.EbeanModule;
import io.jooby.flyway.FlywayModule;
import io.jooby.hikari.HikariModule;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import javax.inject.Named;
import javax.inject.Singleton;
import org.apache.commons.lang3.tuple.Pair;
import org.junit.jupiter.api.Test;

public class UnitTest {

  private static class TestDI extends AbstractModule {
    @Provides
    @Singleton
    @Named("hikariPoolExtension")
    public Extension getHikariExtension() {
      return mock(HikariModule.class);
    }

    @Provides
    @Singleton
    @Named("flyWayMigrationExtension")
    public Extension getFlyWayMigrationExtension() {
      return mock(FlywayModule.class);
    }

    @Provides
    @Singleton
    @Named("ebeanExtension")
    public Extension getEbeanExtension() {
      return mock(EbeanModule.class);
    }

    @Provides
    @Singleton
    @Named("serviceModule")
    public ServiceModule getServiceModule() {
      return mock(ServiceModule.class);
    }

    @Provides
    @Singleton
    @Named("serviceRunningCheckTime")
    public Pair<Integer, TimeUnit> getTime() {
      return Pair.of(0, TimeUnit.MILLISECONDS);
    }
  }

  @Test
  public void shouldSayWelcome() {
    final TestDI testDI = new TestDI();
    final List<Module> abstractModules = new ArrayList<>();
    abstractModules.add(testDI);
    MockRouter router = new MockRouter(new App(Stage.DEVELOPMENT, abstractModules));
    router.get(
        "/",
        rsp -> {
          assertEquals("Hello World!", rsp.value());
          assertEquals(StatusCode.OK, rsp.getStatusCode());
        });
  }
}
