package com.github.polpetta.mezzotre.helper;

import com.github.polpetta.mezzotre.App;
import com.github.polpetta.mezzotre.route.RouteDI;
import com.github.polpetta.mezzotre.util.ServiceModule;
import com.google.common.util.concurrent.Service;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.Stage;
import com.zaxxer.hikari.HikariConfig;
import io.jooby.Extension;
import io.jooby.ebean.EbeanModule;
import io.jooby.flyway.FlywayModule;
import io.jooby.hikari.HikariModule;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import javax.inject.Named;
import javax.inject.Singleton;
import org.apache.commons.lang3.tuple.Pair;
import org.testcontainers.containers.PostgreSQLContainer;

public class IntegrationAppFactory {

  private static class DatabaseDI extends AbstractModule {

    private final PostgreSQLContainer<?> container;

    public DatabaseDI(PostgreSQLContainer<?> container) {
      this.container = container;
    }

    @Provides
    @Singleton
    @Named("hikariPoolExtension")
    public Extension getHikariExtension() throws Exception {
      final Pair<Properties, Properties> propertiesPropertiesPair =
          Loader.loadDefaultEbeanConfigWithPostgresSettings(container);

      return new HikariModule(new HikariConfig(propertiesPropertiesPair.getRight()));
    }

    @Provides
    @Singleton
    @Named("flyWayMigrationExtension")
    public Extension getFlyWayMigrationExtension() {
      return new FlywayModule();
    }

    @Provides
    @Singleton
    @Named("ebeanExtension")
    public Extension getEbeanExtension() {
      return new EbeanModule();
    }

    @Singleton
    @Provides
    @Named("serviceModule")
    public ServiceModule getServiceModule(@Named("services") List<Service> moduleList) {
      return new ServiceModule(moduleList);
    }

    @Provides
    @Singleton
    @Named("serviceRunningCheckTime")
    public Pair<Integer, TimeUnit> getTime() {
      return Pair.of(0, TimeUnit.MILLISECONDS);
    }
  }

  public static App loadCustomDbApplication() {
    final PostgreSQLContainer<?> container =
        new PostgreSQLContainer<>(TestConfig.POSTGRES_DOCKER_IMAGE);
    container.start();
    final RouteDI routeModule = new RouteDI();
    final DatabaseDI databaseDI = new DatabaseDI(container);
    return new App(Stage.DEVELOPMENT, Set.of(databaseDI, routeModule));
  }
}
