package com.github.polpetta.mezzotre.orm.telegram;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.github.polpetta.mezzotre.util.Clock;
import com.github.polpetta.types.json.ChatContext;
import java.util.Collections;
import java.util.Map;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.parallel.Execution;
import org.junit.jupiter.api.parallel.ExecutionMode;
import org.mockito.ArgumentCaptor;

@Execution(ExecutionMode.CONCURRENT)
class ChatUtilTest {
  private Clock fakeClock;
  private ChatUtil chatUtil;

  @BeforeEach
  void setUp() {
    fakeClock = mock(Clock.class);
    chatUtil = new ChatUtil(fakeClock);
  }

  @Test
  void shouldUpdateChatContext() {
    final TgChat fakeTgChat = mock(TgChat.class);
    final ChatContext chatContext = new ChatContext();
    when(fakeTgChat.getChatContext()).thenReturn(chatContext);
    when(fakeClock.now()).thenReturn(69420L);
    chatUtil.updateChatContext(fakeTgChat, "/test1", 42, Collections.emptyMap());

    verify(fakeTgChat, times(1)).getChatContext();
    final ArgumentCaptor<ChatContext> chatContextArgumentCaptor =
        ArgumentCaptor.forClass(ChatContext.class);
    verify(fakeTgChat, times(1)).setChatContext(chatContextArgumentCaptor.capture());
    verify(fakeTgChat, times(0)).setHasHelpBeenShown(eq(true));
    final ChatContext capturedChatContextValue = chatContextArgumentCaptor.getValue();
    assertEquals("/test1", capturedChatContextValue.getStage());
    assertEquals(42, capturedChatContextValue.getStep());
    assertEquals(69420L, capturedChatContextValue.getPreviousMessageUnixTimestampInSeconds());
  }

  @Test
  void shouldAddMapElementsToChatContextToo() {
    final TgChat fakeTgChat = mock(TgChat.class);
    final ChatContext chatContext = new ChatContext();
    when(fakeTgChat.getChatContext()).thenReturn(chatContext);
    when(fakeClock.now()).thenReturn(69420L);

    final Object obj1 = new Object();
    final Object obj2 = new Object();

    chatUtil.updateChatContext(fakeTgChat, "/test1", 42, Map.of("field1", obj1, "field2", obj2));

    verify(fakeTgChat, times(1)).getChatContext();
    final ArgumentCaptor<ChatContext> chatContextArgumentCaptor =
        ArgumentCaptor.forClass(ChatContext.class);
    verify(fakeTgChat, times(1)).setChatContext(chatContextArgumentCaptor.capture());
    final ChatContext capturedChatContextValue = chatContextArgumentCaptor.getValue();
    final Map<String, Object> gotAdditionalProperties =
        capturedChatContextValue.getAdditionalProperties();
    assertEquals(2, gotAdditionalProperties.size());
    assertEquals(obj1, gotAdditionalProperties.get("field1"));
    assertEquals(obj2, gotAdditionalProperties.get("field2"));
  }
}
