package com.github.polpetta.mezzotre.telegram.callbackquery;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import com.github.polpetta.mezzotre.orm.CallbackQueryContextCleaner;
import com.github.polpetta.mezzotre.orm.model.CallbackQueryContext;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.parallel.Execution;
import org.junit.jupiter.api.parallel.ExecutionMode;
import org.slf4j.LoggerFactory;

@Execution(ExecutionMode.CONCURRENT)
class NotFoundTest {

  private CallbackQueryContextCleaner fakeCallbackQueryCleaner;
  private NotFound notFound;

  @BeforeEach
  void setUp() {

    fakeCallbackQueryCleaner = mock(CallbackQueryContextCleaner.class);

    notFound =
        new NotFound(LoggerFactory.getLogger(NotFound.class), fakeCallbackQueryCleaner, "anEvent");
  }

  @Test
  void shouldCallCallbackQueryContextCleaner() throws Exception {
    final CallbackQueryContext fakeCallbackQuery = mock(CallbackQueryContext.class);
    when(fakeCallbackQuery.getId()).thenReturn("anId");
    when(fakeCallbackQuery.getEntryGroup()).thenReturn("123");
    final Update fakeUpdate = mock(Update.class);

    final CompletableFuture<Optional<BaseRequest<?, ?>>> gotResult =
        notFound.process(fakeCallbackQuery, fakeUpdate);

    verify(fakeCallbackQueryCleaner, times(1)).removeGroupAsync(eq("123"));
    assertEquals(Optional.empty(), gotResult.get());
  }
}
