package com.github.polpetta.mezzotre.telegram.callbackquery;

import static org.junit.jupiter.api.Assertions.*;

import com.google.gson.Gson;
import com.pengrad.telegrambot.model.Update;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.parallel.Execution;
import org.junit.jupiter.api.parallel.ExecutionMode;

@Execution(ExecutionMode.CONCURRENT)
class UtilTest {

  private static Gson gson;

  @BeforeAll
  static void beforeAll() {
    gson = new Gson();
  }

  @Test
  void shouldProvideAMessageIdGivenTheUpdate() {
    final Update update =
        gson.fromJson(
            "{\n"
                + "  \"update_id\": 158712614,\n"
                + "  \"callback_query\": {\n"
                + "    \"id\": \"20496049451114620\",\n"
                + "    \"from\": {\n"
                + "      \"id\": 1111111,\n"
                + "      \"is_bot\": false,\n"
                + "      \"first_name\": \"Test Firstname\",\n"
                + "      \"last_name\": \"Test Lastname\",\n"
                + "      \"username\": \"Testusername\",\n"
                + "      \"language_code\": \"en\"\n"
                + "    },\n"
                + "    \"message\": {\n"
                + "      \"message_id\": 2723,\n"
                + "      \"from\": {\n"
                + "        \"id\": 244745330,\n"
                + "        \"is_bot\": true,\n"
                + "        \"first_name\": \"Dev - DavideBot\",\n"
                + "        \"username\": \"devdavidebot\"\n"
                + "      },\n"
                + "      \"date\": 1681218838,\n"
                + "      \"chat\": {\n"
                + "        \"id\": 1111111,\n"
                + "        \"type\": \"private\",\n"
                + "        \"username\": \"Testusername\",\n"
                + "        \"first_name\": \"Test Firstname\",\n"
                + "        \"last_name\": \"Test Lastname\"\n"
                + "      },\n"
                + "      \"text\": \"a message\",\n"
                + "      \"reply_markup\": {\n"
                + "        \"inline_keyboard\": [\n"
                + "          [\n"
                + "            {\n"
                + "              \"text\": \"English\",\n"
                + "              \"callback_data\": \"9a64be11-d086-4bd9-859f-720c43dedcb5\"\n"
                + "            },\n"
                + "            {\n"
                + "              \"text\": \"Italian\",\n"
                + "              \"callback_data\": \"8768d660-f05f-4f4b-bda5-3451ab573d56\"\n"
                + "            }\n"
                + "          ]\n"
                + "        ]\n"
                + "      }\n"
                + "    }\n"
                + "  }\n"
                + "}",
            Update.class);

    assertEquals(
        2723, Util.extractMessageId(update).get(), "A message id should be returned, 2723");
  }

  @Test
  void shouldGiveAnEmptyOptionalWithoutMessageId() {
    final Update update =
        gson.fromJson(
            "{\n"
                + "  \"update_id\": 158712614,\n"
                + "  \"callback_query\": {\n"
                + "    \"id\": \"20496049451114620\",\n"
                + "    \"from\": {\n"
                + "      \"id\": 1111111,\n"
                + "      \"is_bot\": false,\n"
                + "      \"first_name\": \"Test Firstname\",\n"
                + "      \"last_name\": \"Test Lastname\",\n"
                + "      \"username\": \"Testusername\",\n"
                + "      \"language_code\": \"en\"\n"
                + "    }\n"
                + "  }\n"
                + "}",
            Update.class);

    assertTrue(Util.extractMessageId(update).isEmpty(), "The shouldn't be any message id");
  }
}
